// Contains a base class for Kalman predictors


#ifndef KALMANBASE_H
#define KALMANBASE_H

#include "mmatrix.H"
#include "../pose.H"
#include "kalmanUpdate.H"


#define MAXBEGIN 16
#define INITVAR 100

// base class for Kalman Filter predictors
// does the basic kalman filter routine
class KalmanBase 
{
  public:
    KalmanBase() { }
    virtual ~KalmanBase() { }

    void kalmanFilter(Pose *userpos, int zNum, int iNum, KALMANMODEL mod);
  protected:

  // hold for all state vectors
    MMatrix  _HH;
    MMatrix  _QQ;
    MMatrix  _RR;
    MMatrix  _fundMat;

  // unique for each position component
    MMatrix  _PP[3]; 
    MMatrix  _state[3]; 
};



// base class for the Extended Kalman Filter predictors
class ExtendedKalmanBase
{
  public:
    ExtendedKalmanBase() 
    { _up = new Updater(); }
    virtual ~ExtendedKalmanBase() 
    { delete _up; }

    void extendedKalmanFilter(Pose *userpos, int zNum, int iNum, KALMANMODEL mod);

  protected:
    MMatrix _EH;
    MMatrix _EQ;
    MMatrix _ER;
    MMatrix _EfundMat;
    MMatrix _EP;
    MMatrix _Estate;

  // since things have to be recomputed for every k we need to hold a few things
   double _edt;
   double _spd;
  
   Updater *_up;
   
};


// base class for Unscented Kalman Filter predictors
class UnscentedKalmanBase
{
  public: 
    UnscentedKalmanBase() 
    { _uup = new Updater(); }
    virtual ~UnscentedKalmanBase() 
    { delete _uup; }

    void unscentedKalmanFilter(Pose *userpos, int zNum, KALMANMODEL mod);
    Quat sigmaPredict(double pinterval, KALMANMODEL mod);

  protected:
    MMatrix _UQ;
    MMatrix _UR;
    MMatrix _UP;
    MMatrix _Ustate;

    double _udt;  
    double _uspd;  
    int    _dim;

    Updater *_uup;

};

#endif
