// Joseph J LaViola Jr.
// Basic vector class

#ifndef VECTOR_H
#define VECTOR_H

#include <math.h>
#include <stdio.h>
#include <iostream.h>
#include <assert.h>

class VVector 
{
  protected:
    double          *_vec;
    int              _size;

  public:
    VVector(int size, double *vals) : _size(size) {
      _vec = new double[size]; 
      for (int i = 0; i < size; ++i)
	_vec[i] = vals[i];    
    }

    VVector(int size) : _size(size) {  
      _vec = new double[size];
      for (int i = 0; i < size; ++i) 
	_vec[i]= 0.0;    
    }

    VVector(int size, double val) : _size(size) {
      _vec = new double[size];
      for (int i = 0; i < size; ++i)
	_vec[i] = val;
    }
  
    VVector(int size, VVector *v) : _size(size) {
      _vec = new double[size];
      for (int i = 0; i < size; ++i)
	_vec[i] = (*v)[i];
    }
    
    VVector(VVector *p1, VVector *p2) {
      assert(p1->getSize() == p2->getSize());
      _size = p1->getSize(); 
      _vec = new double[_size];
      for (int i = 0; i < _size; ++i)
         _vec[i] = (*p2)[i] - (*p1)[i];
    }

    // destructor
    virtual ~VVector() { delete [] _vec; }
    

    void writeOver(VVector *vec) {
      assert(vec->getSize() == _size);
      for (int i = 0; i < _size; ++i)
       _vec[i] = (*vec)[i];
    }

    

    int getSize() { return _size; } 
    double * getVector() { return _vec;}  
  
    double   operator [] (int i) const { return _vec[i];}  
    double & operator [] (int i) { return _vec[i];}

    void scalarMultiply(double x) {
      for (int i = 0; i < _size; ++i)
	_vec[i] *= x;
    }

    double length() {  // computes the length of the vector
      double sum = 0.0;
      for (int i = 0; i < _size; ++i)
	sum += _vec[i] * _vec[i];
      return sqrt(sum);
    } 
  
    VVector * norm() {  // returns the normalized vector
      double len = length();
      double *vals = new double[_size];
      for (int i = 0; i < _size; ++i)
         vals[i] = _vec[i] / len;
      VVector *v = new VVector(_size, vals);
      delete [] vals;
      return v;
    } 

    double dist(VVector *v) { // computes the distance beteen two points
      if (v->getSize() != _size) {
        cerr << "Points are of unequal dimension, cannot compute distance." << endl;
        return -1.0;
      }
      double sum = 0.0;
      for (int i = 0; i < _size; ++i)
	sum += pow((*v)[i] - _vec[i], 2);
      return sqrt(sum);
    }

    void printVector() {
      //      cerr << "------------Vector--------------" << endl; 
      for (int i = 0; i < _size; ++i)
	cerr << " " << _vec[i] << ", ";
      cerr << endl;
  }
  

};

#endif
