// member functions for Results object

#include "results.H"

#include "ui.H"
#include "dispenser.H"
#include "pose.H"

#include <qstatusbar.h>
#include <qmultilineedit.h>
#include <qlabel.h>
#include <qstring.h>
#include <iostream.h>
#include <stdio.h>

#define MAX 100

///////////////////
//NOTE:: ALL POSITION ERROR REPORTS ARE IN INCHES.  THIS REQUIRES
// A CONVERSTION.  ALWAYS MULTIPLY BY 12.0 TO GET FROM FEET TO INCHES


// functions for the Results object
Results::Results(GlobalWidget *gbw) : _preclist(MAX), _ureclist(MAX), _errlistpct(MAX),
                                      _errlistrms(MAX), _errlistinf(MAX), _gbw(gbw)
{

 _etype = _gbw->getErrorType();
 _ttype = _gbw->getTestType();

}

// destructor
Results::~Results()
{
 for (int i = 0; i < _preclist.num(); ++i) 
   delete _preclist[i];
}


void
Results::reload() 
{
  _etype = _gbw->getErrorType();
  _ttype = _gbw->getTestType();
}

// clear data structures
void
Results::clear()
{
  int i;
  for (i = 0; i < _preclist.num(); ++i) 
    delete _preclist[i];
 _preclist.clear();
 _ureclist.clear();

 _errlistrms.clear();
 _errlistinf.clear();
 _errlistpct.clear();

}

void
Results::clearPRec()
{ 
  for (int i = 0; i < _preclist.num(); ++i)
    delete _preclist[i];
  _preclist.clear();
}



// this function examines the error type and test type and 
// calls the appropriate analysis function
void
Results::errorAnalysis()
{
  _gbw->getStatusBar()->message("Comupting results...");
  // switch on the test type
  switch(_ttype) {
    case tSingle: {   
      displaySingleError();
      displaySingleGraphics();
      break;
    }
    case tRunParams: {
      computeSingleError();
      break;
    }
    case tRunSP: {
      computeSingleError();
      break;
    }
    case tRunPI: {
      computeSingleError();
      break;
    }
    case tRunN: {
     computeSingleError();
     break;
    }
    case tRunAll: {
    }
    default:
      cerr << "We have a problem in Results::errorAnalysis!!!" << endl;
  }
  // _gbw->getStatusBar()->message("Results Displayed.");
}


// display plots


void 
Results::displaySingleGraphics()
{
  if (!_gbw->getPlotsDialog()->plot())  // no plots 
    return;
  
  // first clear everything
  MATLABEngine *eng = _gbw->getMEngine();
  eng->runCmd(QString("clear"));
	    
  // create large figure and position it
  eng->runCmd(QString("figure(1)"));
  eng->runCmd(QString("set(1, 'Position', [470 -35 800 980])"));
 
  int numtrue = _ureclist[0]->getPList().num();
  int numpred = _preclist[0]->getPredList().num();
  int numcorr = _preclist[0]->getCorruptList().num();
  

  // we know we are going to need the first vector in the user motion rec
  double  *tt = new double[numtrue];
  for (int i = 0; i < numtrue; ++i)
    tt[i] = _ureclist[0]->getPList()[i]->_tstamp;
  mxArray *TT = mxCreateDoubleMatrix(1, numtrue, mxREAL);
  mxSetName(TT,"TT");
  memcpy((void*)mxGetPr(TT), (void*)tt, numtrue*sizeof(double));
  eng->sendArray(TT); 
  mxDestroyArray(TT);
  delete [] tt;

  // plot predicted vs actual for each component the users requested
  if (_gbw->getPlotsDialog()->getX()) {
    // load in the data for the x plot 
    double *xt = new double[numtrue];
    for (int i = 0; i < numtrue; ++i)
      xt[i] = _ureclist[0]->getPList()[i]->_pos[0];
    double *xp = new double[numpred];
    for (int i = 0; i < numpred; ++i) 
      xp[i] = _preclist[0]->getPredList()[i]->_pos[0];
    double *xc = new double[numcorr];
    for (int i = 0; i < numcorr; ++i) 
      xc[i] = _preclist[0]->getCorruptList()[i]->_pos[0];  

    plotSignal("XT", "XP", "XC", "subplot(4,2,1)", "plot(TT,XT)",
               "stairs(TP,XP,'r-')", "plot(TP,XP,'r.')", "plot(TC,XC,'k-')",
               "title('Predicted X vs. True X')",
               "xlabel('Time (ms)')", "ylabel('X')", numpred, numtrue, numcorr,
                xt,xp, xc);
    delete [] xt;
    delete [] xp;
    delete [] xc;
  }
  if (_gbw->getPlotsDialog()->getY()) {
    // plot data for y 
    double *yt = new double[numtrue];
    for (int i = 0; i < numtrue; ++i)
      yt[i] = _ureclist[0]->getPList()[i]->_pos[1];
    double *yp = new double[numpred];
    for (int i = 0; i < numpred; ++i) 
      yp[i] = _preclist[0]->getPredList()[i]->_pos[1];
    double *yc = new double[numcorr];
    for (int i = 0; i < numcorr; ++i) 
      yc[i] = _preclist[0]->getCorruptList()[i]->_pos[1];

    plotSignal("YT", "YP", "YC", "subplot(4,2,3)", "plot(TT,YT)",
               "stairs(TP,YP,'r-')", "plot(TP,YP,'r.')", "plot(TC,YC,'k-')",
               "title('Predicted Y vs. True Y')",
               "xlabel('Time (ms)')", "ylabel('Y')", numpred, numtrue,numcorr,
	       yt,yp,yc);
    delete [] yt;
    delete [] yp;
    delete [] yc;
  }
  if (_gbw->getPlotsDialog()->getZ()) {
    // plot data for z
    double *zt = new double[numtrue];
    for (int i = 0; i < numtrue; ++i)
      zt[i] = _ureclist[0]->getPList()[i]->_pos[2];
    double *zp = new double[numpred];
    for (int i = 0; i < numpred; ++i) 
      zp[i] = _preclist[0]->getPredList()[i]->_pos[2];
    double *zc = new double[numcorr];
    for (int i = 0; i < numcorr; ++i) 
      zc[i] = _preclist[0]->getCorruptList()[i]->_pos[2];
    
    plotSignal("ZT", "ZP", "ZC", "subplot(4,2,5)", "plot(TT,ZT)",
               "stairs(TP,ZP,'r-')", "plot(TP,ZP,'r.')", "plot(TC,ZC,'k-')",
               "title('Predicted Z vs. True Z')",
               "xlabel('Time (ms)')", "ylabel('Z')", numpred, numtrue,numcorr,
	       zt,zp,zc);
    delete [] zt;
    delete [] zp;
    delete [] zc;

  }
  if (_gbw->getPlotsDialog()->getQX()) {
    // plot data for qx
    double *qxt = new double[numtrue];
    for (int i = 0; i < numtrue; ++i)
      qxt[i] = _ureclist[0]->getPList()[i]->_orient.V()[0];
    double *qxp = new double[numpred];
    for (int i = 0; i < numpred; ++i) 
      qxp[i] = _preclist[0]->getPredList()[i]->_orient.V()[0];
    double *qxc = new double[numcorr];
    for (int i = 0; i < numcorr; ++i) 
      qxc[i] = _preclist[0]->getCorruptList()[i]->_orient.V()[0];

    plotSignal("QXT", "QXP", "QXC", "subplot(4,2,2)", "plot(TT,QXT)",
               "stairs(TP,QXP,'r-')", "plot(TP,QXP,'r.')", "plot(TC,QXC,'k-')",
               "title('Predicted QX vs. True QX')",
               "xlabel('Time (ms)')", "ylabel('QX')", numpred, numtrue, numcorr,
	       qxt,qxp,qxc);
    delete [] qxt;
    delete [] qxp;
    delete [] qxc;

  } 
  if (_gbw->getPlotsDialog()->getQY()) {
    // plot data for qy
    double *qyt = new double[numtrue];
    for (int i = 0; i < numtrue; ++i)
      qyt[i] = _ureclist[0]->getPList()[i]->_orient.V()[1];
    double *qyp = new double[numpred];
    for (int i = 0; i < numpred; ++i) 
      qyp[i] = _preclist[0]->getPredList()[i]->_orient.V()[1];
    double *qyc = new double[numcorr];
    for (int i = 0; i < numcorr; ++i) 
      qyc[i] = _preclist[0]->getCorruptList()[i]->_orient.V()[1];

    plotSignal("QYT", "QYP", "QYC", "subplot(4,2,4)", "plot(TT,QYT)",
               "stairs(TP,QYP,'r-')", "plot(TP,QYP,'r.')", "plot(TC,QYC,'k-')",
               "title('Predicted QY vs. True QY')",
               "xlabel('Time (ms)')", "ylabel('QY')", numpred, numtrue,numcorr,
	       qyt,qyp,qyc);
    delete [] qyt;
    delete [] qyp;
    delete [] qyc;
  } 
  if (_gbw->getPlotsDialog()->getQZ()) {
    // plot data for qz
    double *qzt = new double[numtrue];
    for (int i = 0; i < numtrue; ++i)
      qzt[i] = _ureclist[0]->getPList()[i]->_orient.V()[2];
    double *qzp = new double[numpred];
    for (int i = 0; i < numpred; ++i) 
      qzp[i] = _preclist[0]->getPredList()[i]->_orient.V()[2];
    double *qzc = new double[numcorr];
    for (int i = 0; i < numcorr; ++i) 
      qzc[i] = _preclist[0]->getCorruptList()[i]->_orient.V()[2];
   
    plotSignal("QZT", "QZP", "QZC", "subplot(4,2,6)", "plot(TT,QZT)",
               "stairs(TP,QZP,'r-')", "plot(TP,QZP,'r.')", "plot(TC,QZC,'k-')",
               "title('Predicted QZ vs. True QZ')",
               "xlabel('Time (ms)')", "ylabel('QZ')", numpred, numtrue,numcorr,
	       qzt,qzp,qzc);
    delete [] qzt;
    delete [] qzp;
    delete [] qzc;
  } 
  if (_gbw->getPlotsDialog()->getQW()) {
    // plot data for qw
    double *qwt = new double[numtrue];
    for (int i = 0; i < numtrue; ++i)
      qwt[i] = _ureclist[0]->getPList()[i]->_orient.S();
    double *qwp = new double[numpred];
    for (int i = 0; i < numpred; ++i) 
      qwp[i] = _preclist[0]->getPredList()[i]->_orient.S();
    double *qwc = new double[numcorr];
    for (int i = 0; i < numcorr; ++i) 
      qwc[i] = _preclist[0]->getCorruptList()[i]->_orient.S();

    plotSignal("QWT", "QWP", "QWC", "subplot(4,2,8)", "plot(TT,QWT)",
               "stairs(TP,QWP,'r-')", "plot(TP,QWP,'r.')", "plot(TC,QWC,'k-')",
               "title('Predicted QW vs. True QW')",
               "xlabel('Time (ms)')", "ylabel('QW')", numpred, numtrue, numcorr,
	       qwt,qwp,qwc);
    delete [] qwt;
    delete [] qwp;
    delete [] qwc;
  } 

  // this funtion will display error graphs for each signal
  plotErrorGraphs();

}

void
Results::plotErrorGraphs()
{

  // first clear everything
  MATLABEngine *eng = _gbw->getMEngine();
  eng->runCmd(QString("clear"));
	    
  // create large figure and position it
  eng->runCmd(QString("figure(2)"));
  eng->runCmd(QString("set(2, 'Position', [470 -10 800 980])"));

  int numtrue = _preclist[0]->getTrueList().num();

  // we know we are going to need the first vector in the pred motion rec
  double  *tt = new double[numtrue];
  for (int i = 0; i < numtrue; ++i) 
    tt[i] = _preclist[0]->getTrueList()[i]->_tstamp;
  mxArray *TT = mxCreateDoubleMatrix(1, numtrue, mxREAL);
  mxSetName(TT,"TT");
  memcpy((void*)mxGetPr(TT), (void*)tt, numtrue*sizeof(double));
  eng->sendArray(TT); 
  mxDestroyArray(TT);
  delete [] tt;

  // we need to create a list of differences (true - predicted)
  FIGarray<Pose *> diffs(numtrue); 
  
  for (int i = 0; i < numtrue; ++i) {
    Pose *tmp = new Pose();
    tmp->_pos = _preclist[0]->getTrueList()[i]->_pos -
                _preclist[0]->getPredList()[i]->_pos;
    tmp->_orient = _preclist[0]->getTrueList()[i]->_orient -
                   _preclist[0]->getPredList()[i]->_orient;

    diffs += tmp;
  }
 
  // now we send the info to matlab and plot
   if (_gbw->getPlotsDialog()->getX()) {
     double *dx = new double[numtrue];
     for (int i = 0; i < numtrue; ++i) 
       dx[i] = diffs[i]->_pos[0];  
     plotError("DX",  "subplot(4,2,1)", "plot(TT,DX,'r-')",
               "title('X Error')",
               "xlabel('Time (ms)')", "ylabel('X')", numtrue, dx);
     delete [] dx;
   }

   if (_gbw->getPlotsDialog()->getY()) {
     double *dy = new double[numtrue];
     for (int i = 0; i < numtrue; ++i)
       dy[i] = diffs[i]->_pos[1];
     plotError("DY",  "subplot(4,2,3)", "plot(TT,DY,'r-')",
               "title('Y Error')",
               "xlabel('Time (ms)')", "ylabel('Y')", numtrue, dy);
     delete [] dy;
   }
   
    if (_gbw->getPlotsDialog()->getZ()) {
     double *dz = new double[numtrue];
     for (int i = 0; i < numtrue; ++i)
       dz[i] = diffs[i]->_pos[2];
     plotError("DZ",  "subplot(4,2,5)", "plot(TT,DZ,'r-')",
               "title('Z Error')",
               "xlabel('Time (ms)')", "ylabel('Z')", numtrue, dz);
     delete [] dz;
   }
   
    if (_gbw->getPlotsDialog()->getQX()) {
     double *dqx = new double[numtrue];
     for (int i = 0; i < numtrue; ++i)
       dqx[i] = diffs[i]->_orient.V()[0];
     plotError("DQX",  "subplot(4,2,2)", "plot(TT,DQX,'r-')",
               "title('QX Error')",
               "xlabel('Time (ms)')", "ylabel('QX')", numtrue, dqx);
     delete [] dqx;
   }

   if (_gbw->getPlotsDialog()->getQY()) {
     double *dqy = new double[numtrue];
     for (int i = 0; i < numtrue; ++i)
       dqy[i] = diffs[i]->_orient.V()[1];
     plotError("DQY",  "subplot(4,2,4)", "plot(TT,DQY,'r-')",
               "title('QY Error')",
               "xlabel('Time (ms)')", "ylabel('QY')", numtrue, dqy);
     delete [] dqy;
   }

    if (_gbw->getPlotsDialog()->getQZ()) {
     double *dqz = new double[numtrue];
     for (int i = 0; i < numtrue; ++i)
       dqz[i] = diffs[i]->_orient.V()[2];
     plotError("DQZ",  "subplot(4,2,6)", "plot(TT,DQZ,'r-')",
               "title('QZ Error')",
               "xlabel('Time (ms)')", "ylabel('QZ')", numtrue, dqz);
     delete [] dqz;
   }
   
   if (_gbw->getPlotsDialog()->getQW()) {
     double *dqw = new double[numtrue];
     for (int i = 0; i < numtrue; ++i)
       dqw[i] = diffs[i]->_orient.S();
     plotError("DQW",  "subplot(4,2,8)", "plot(TT,DQW,'r-')",
               "title('QZ Error')",
               "xlabel('Time (ms)')", "ylabel('QW')", numtrue, dqw);
     delete [] dqw;
   }
   
 int i;
  for (i = 0; i < diffs.num(); ++i) 
    delete diffs[i];
}

void 
Results::plotError(const char *name, const char *subplot, const char *plot,
                   const char *title, const char *xlabel, const char *ylabel,
                   int num, double *diff)
{
  MATLABEngine *eng = _gbw->getMEngine();
  mxArray *SIGT =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxSetName(SIGT,name);
  memcpy((void*)mxGetPr(SIGT), (void*)diff, num*sizeof(double));
  eng->sendArray(SIGT); 
  mxDestroyArray(SIGT);

  eng->runCmd(QString(subplot));
  eng->runCmd(QString(plot));
  eng->runCmd(QString(title));
  eng->runCmd(QString(xlabel));
  eng->runCmd(QString(ylabel));
}

void
Results::plotSignal(const char *trues, const char *preds, const char *corrs,
                    const char *subplot,
                    const char *plottrue, const char *plotpred, const char *plotdot, 
                    const char *corrsig, const char *title, 
                    const char *xlabel,
                    const char *ylabel, int numpred, int numtrue, int numcorr,
                    double *truesig, double *predsig, double *corrupted)
{

  MATLABEngine *eng = _gbw->getMEngine();
  mxArray *SIGT =  mxCreateDoubleMatrix(1, numtrue, mxREAL);
  mxSetName(SIGT,trues);
  memcpy((void*)mxGetPr(SIGT), (void*)truesig, numtrue*sizeof(double));
  eng->sendArray(SIGT); 
  mxDestroyArray(SIGT);

  double *tp = new double[numpred];
  for (int i = 0; i < numpred; ++i) 
    tp[i] = _preclist[0]->getPredList()[i]->_tstamp;

  mxArray *SIGP =  mxCreateDoubleMatrix(1, numpred, mxREAL);
  mxSetName(SIGP,preds);
  memcpy((void*)mxGetPr(SIGP), (void*)predsig, numpred*sizeof(double));

  mxArray *TP =  mxCreateDoubleMatrix(1, numpred, mxREAL);
  mxSetName(TP,"TP");
  memcpy((void*)mxGetPr(TP), (void*)tp, numpred*sizeof(double));
  eng->sendArray(SIGP); 
  eng->sendArray(TP); 
  mxDestroyArray(SIGP);
  mxDestroyArray(TP);
  delete [] tp;

  // get the time values for for the corrupted signal
  double *tc = new double[numcorr];
  for (int i = 0; i < numcorr; ++i) 
    tc[i] = _preclist[0]->getCorruptList()[i]->_tstamp;

  mxArray *TC =  mxCreateDoubleMatrix(1, numcorr, mxREAL);
  mxSetName(TC,"TC");
  memcpy((void*)mxGetPr(TC), (void*)tc, numcorr*sizeof(double));
  eng->sendArray(TC); 
  mxDestroyArray(TC);
  delete [] tc; 
   
  // get corrupted signal
  mxArray *SIGC =  mxCreateDoubleMatrix(1, numcorr, mxREAL);
  mxSetName(SIGC,corrs);
  memcpy((void*)mxGetPr(SIGC), (void*)corrupted, numcorr*sizeof(double));
  eng->sendArray(SIGC); 
  mxDestroyArray(SIGC);

  eng->runCmd(QString(subplot));
  eng->runCmd(QString(plottrue));
  eng->runCmd(QString("hold on"));
  eng->runCmd(QString(corrsig)); 
  eng->runCmd(QString(plotpred)); 

  if (getenv("SHOW_PLOT_POINTS"))
    eng->runCmd(QString(plotdot)); 
  
  eng->runCmd(QString(title));
  eng->runCmd(QString(xlabel));
  eng->runCmd(QString(ylabel));
  eng->runCmd(QString("hold off"));
} 

// compute and display error depending on error type
void
Results::displaySingleError()
{
  // switch on error type
  switch(_etype) {
    case tRootMSE: {
     printRootMSE(rootMSE(0));
     break;
    }
    case tLInfinity: {
     printLInf(lInfinity(0));
     break;
    }
    case tPctBetter: {
     printPB(pctBetter(0));
     break;
    }
    case tChooseAll: {
     printRootMSE(rootMSE(0));
     printLInf(lInfinity(0));
     printPB(pctBetter(0));
     break;
    }
  default:
    cerr << "We have a problem in Results::displaySingleError()!!!" << endl;
  }
}


void
Results::computeSingleError()
{
  // switch on error type
  switch(_etype) {
    case tRootMSE: {
     _errlistrms += rootMSE(0);
     break;
    }
    case tLInfinity: {
     _errlistinf += lInfinity(0);
     break;
    }
    case tPctBetter: {
      _errlistpct += pctBetter(0);
      break;
    }
    case tChooseAll: {
      _errlistrms += rootMSE(0); 
      _errlistinf += lInfinity(0);
      _errlistpct += pctBetter(0);
     break;
    }
  default:
    cerr << "We have a problem in Results::computeSingleError()!!!" << endl;
  }


}


// error metric functions
ErrorNum
Results::rootMSE(int i)
{
  ErrorNum err;
  
  // compute the rootMSE and display the results for the ith pred record
  if (i > _preclist.num()-1) {
    cerr << "Bad Index in Results::rootMSE" << endl;
    return err;
  }


  // we compute both global and component wise Root MSE
  // first global
  double sumpos = 0.0;
  double sumrot = 0.0;
  for (int j = 0; j < _preclist[i]->getPredList().num(); ++j) {
    double len = distance1(_preclist[i]->getPredList()[j]->_pos,
                         _preclist[i]->getTrueList()[j]->_pos);
    sumpos += len*len;
    // now for rotation
    Quat qdiff = _preclist[i]->getTrueList()[j]->_orient * 
                 inverse(_preclist[i]->getPredList()[j]->_orient);
    double ang = (360.0 / M_PI) * acos(qdiff.S());
    sumrot += ang*ang;
  }
  err._gpos = sqrt(sumpos / _preclist[i]->getPredList().num());
  err._grot = sqrt(sumrot / _preclist[i]->getPredList().num());

  // now for component wise rmse
  Vec3d sumpv(0.0);
  Quat  sumrv(0,0,0,0);
  for (int j = 0; j < _preclist[i]->getPredList().num(); ++j) {
    Vec3d tmp  = _preclist[i]->getPredList()[j]->_pos - 
                 _preclist[i]->getTrueList()[j]->_pos;
    
    sumpv += tmp*tmp;
   
    Quat tmp2 =  _preclist[i]->getPredList()[j]->_orient - 
                 _preclist[i]->getTrueList()[j]->_orient;

    sumrv += specialmultiply(tmp2,tmp2);
  }
  err._lpos = mysqrt((1.0 / _preclist[i]->getPredList().num())* sumpv);
  err._lrot = mysqrt((1.0 / _preclist[i]->getPredList().num())* sumrv);

  return err;
}


ErrorNum
Results::lInfinity(int i)
{
 ErrorNum err;

 // compute the max norm and display the results for the ith pred record
  if (i > _preclist.num()-1) {
    cerr << "Bad Index in Results::lInfinity" << endl;
    return err;
  }

   // compute global and component wise max norm
  double possum = 0.0;
  double rotsum = 0.0;

  for (int j = 0; j < _preclist[i]->getPredList().num(); ++j) {
    possum = distance1(_preclist[i]->getPredList()[j]->_pos,
                         _preclist[i]->getTrueList()[j]->_pos);
   
    if (possum > err._gpos)
      err._gpos = possum;
    
    Quat qdiff = _preclist[i]->getTrueList()[j]->_orient * 
                 inverse(_preclist[i]->getPredList()[j]->_orient);
    rotsum = fabs((360.0 / M_PI) * acos(qdiff.S()));
    
    if (rotsum > err._grot)
      err._grot = rotsum;
  }
  
  // now for component wise
  for (int j = 0; j < _preclist[i]->getPredList().num(); ++j) {
    Vec3d tmp  = myfabs(_preclist[i]->getPredList()[j]->_pos,
                 _preclist[i]->getTrueList()[j]->_pos);
    if (tmp[0] > err._lpos[0])
      err._lpos[0] = tmp[0];
    if (tmp[1] > err._lpos[1])
      err._lpos[1] = tmp[1];
    if (tmp[2] > err._lpos[2])
      err._lpos[2] = tmp[2];    
        
    Quat tmp2 =  myfabs(_preclist[i]->getPredList()[j]->_orient,
                 _preclist[i]->getTrueList()[j]->_orient);
    if (tmp2.V()[0] > err._lrot.V()[0])
      err._lrot.V()[0] = tmp2.V()[0];
    if (tmp2.V()[1] > err._lrot.V()[1])
      err._lrot.V()[1] = tmp2.V()[1];
    if (tmp2.V()[2] > err._lrot.V()[2])
      err._lrot.V()[2] = tmp2.V()[2];
    if (tmp2.S() > err._lrot.S())
      err._lrot.S() = tmp2.S();
  }
 
 return err;
  
}


// computes percentage of how better predictor is against random-walk forecast
ErrorNum
Results::pctBetter(int i)
{
  ErrorNum err;

  // compute the max norm and display the results for the ith pred record
  if (i > _preclist.num()-1) {
    cerr << "Bad Index in Results::pctBetter" << endl;
    return err;
  }

  int gposc = 0;
  int grotc = 0;
  int poscnt[3] = {0,0,0};
  int quatcnt[4] = {0,0,0,0};
 
  // count how many times the current predictor performed better than
  // the random-walk forecast

  for (int j = 0; j < _preclist[i]->getPredList().num(); ++j) {
    for (int k = 0; k < 3; ++k) {
      if (fabs(_preclist[i]->getPredList()[j]->_pos[k] -
               _preclist[i]->getTrueList()[j]->_pos[k]) <
          fabs(_preclist[i]->getRWalkList()[j]->_pos[k] - 
               _preclist[i]->getTrueList()[j]->_pos[k]))
        ++poscnt[k];
      if (fabs(_preclist[i]->getPredList()[j]->_orient.V()[k] -
	       _preclist[i]->getTrueList()[j]->_orient.V()[k]) <
          fabs(_preclist[i]->getRWalkList()[j]->_orient.V()[k] -
	       _preclist[i]->getTrueList()[j]->_orient.V()[k]))
        ++quatcnt[k];
    }
    if (fabs(_preclist[i]->getPredList()[j]->_orient.S() -
              _preclist[i]->getTrueList()[j]->_orient.S()) <
	fabs(_preclist[i]->getRWalkList()[j]->_orient.S() - 
	     _preclist[i]->getTrueList()[j]->_orient.S()))
      ++quatcnt[3];

    // global pos and rot
    double distc = distance1(_preclist[i]->getPredList()[j]->_pos,
                         _preclist[i]->getTrueList()[j]->_pos);
    double distrw = distance1(_preclist[i]->getRWalkList()[j]->_pos,
                         _preclist[i]->getTrueList()[j]->_pos);
    if (distc < distrw)
     ++gposc;

    Quat qdiff = _preclist[i]->getTrueList()[j]->_orient * 
                 inverse(_preclist[i]->getPredList()[j]->_orient);
    double rotc = fabs((360.0 / M_PI) * acos(qdiff.S())); 
   
    Quat qdiff2 = _preclist[i]->getTrueList()[j]->_orient * 
                  inverse(_preclist[i]->getRWalkList()[j]->_orient);
    double rotrw = fabs((360.0 / M_PI) * acos(qdiff2.S())); 

    if (rotc < rotrw)
     ++grotc;
    
  }

  // now compute percentages from the counts
  for (int j = 0; j < 3; ++j) {
     err._lpos[j] = 100.0 * ((double)poscnt[j] / _preclist[i]->getPredList().num());
     err._lrot.V()[j] = 100.0 * ((double)quatcnt[j] / 
                        _preclist[i]->getPredList().num());
  }
  err._lrot.S() = 100.0 * ((double)quatcnt[3] / _preclist[i]->getPredList().num());

  err._gpos = 100.0 * ((double)gposc / _preclist[i]->getPredList().num());
  err._grot = 100.0 * ((double)grotc / _preclist[i]->getPredList().num());
  return err;
}



void
Results::printBasicMaterial(int nopint)
{
  if (_gbw->getPredType() == tLocalPoly) {
    _gbw->getTextBox()->append("Predictor: " + _gbw->getCPanel()->getCurPredictor()->text() +" - Algo " + QString::number((int)_gbw->getLPolyDialog()->getLPType()));
    _gbw->getTextBox()->append("Window Size: " + QString::number(_gbw->getLPolyDialog()->getPWindow()) + " Degree: " + QString::number(_gbw->getLPolyDialog()->getPDegree()));
  }

  else if (_gbw->getPredType() == tExpSmooth) {
    _gbw->getTextBox()->append("Predictor: " + _gbw->getCPanel()->getCurPredictor()->text() +" - Algo " + QString::number((int)_gbw->getExpSmDialog()->getExpSmoothType()));
    _gbw->getTextBox()->append("Alpha: " + QString::number(_gbw->getExpSmDialog()->getAlpha()) + " Beta: " + QString::number(_gbw->getExpSmDialog()->getBeta()));
  
  }
  else if (_gbw->getPredType() == tKalman) {
   if (_gbw->getKalmanPredDialog()->getUKFFlag())
    _gbw->getTextBox()->append("Predictor: " + _gbw->getCPanel()->getCurPredictor()->text() +" (UKF)" + " - Algo " + QString::number((int)_gbw->getKalmanPredDialog()->getKalmanType()));  
   else
     _gbw->getTextBox()->append("Predictor: " + _gbw->getCPanel()->getCurPredictor()->text() +" (EKF)" + " - Algo " + QString::number((int)_gbw->getKalmanPredDialog()->getKalmanType()));  
  }
  else
    _gbw->getTextBox()->append("Predictor: " + _gbw->getCPanel()->getCurPredictor()->text());


  if (nopint == 0)
  _gbw->getTextBox()->append("Sampling Rate: " +  _gbw->getCPanel()->getCurSP()->text() +
                             "     Precdiction Interval: " +
                             _gbw->getCPanel()->getCurPredInt()->text() +
                             "      Noise Variance: " +
                             QString::number(_gbw->getCPanel()->getNoiseVar(), 'g',8));
  else if (nopint == 1)
   _gbw->getTextBox()->append("Sampling Rate: " +  _gbw->getCPanel()->getCurSP()->text() +
                             "      Noise Variance: " +
                             QString::number(_gbw->getCPanel()->getNoiseVar(), 'g',8));
  else if (nopint == 2)
    _gbw->getTextBox()->append("Sampling Rate: " +  _gbw->getCPanel()->getCurSP()->text() +
                             "     Precdiction Interval: " +
                             _gbw->getCPanel()->getCurPredInt()->text());
}


void 
Results::printSingleReport(ErrorNum err)
{
  _gbw->getTextBox()->append("Postion(inches): " + QString::number(12.0*err._gpos,'g',8));  
  _gbw->getTextBox()->append("Component Wise - ");
  _gbw->getTextBox()->append("                 X: " + QString::number(12.0*err._lpos[0],'g',8)); 
  _gbw->getTextBox()->append("                 Y: " + QString::number(12.0*err._lpos[1],'g',8));
  _gbw->getTextBox()->append("                 Z: " + QString::number(12.0*err._lpos[2],'g',8));
   _gbw->getTextBox()->append("    ");
  _gbw->getTextBox()->append("Orientation(degrees): " + QString::number(err._grot,'g',8));  
  _gbw->getTextBox()->append("Component Wise - "); 
  _gbw->getTextBox()->append( "                 QX: " + QString::number(err._lrot.V()[0],'g',8));
  _gbw->getTextBox()->append("                 QY: " + QString::number(err._lrot.V()[1],'g',8));  
  _gbw->getTextBox()->append("                 QZ: " + QString::number(err._lrot.V()[2],'g',8));
  _gbw->getTextBox()->append("                 QW: " + QString::number(err._lrot.S(),'g',8));
  _gbw->getTextBox()->append("    ");

}

void
Results::printPctReport(ErrorNum err)
{
  _gbw->getTextBox()->append("Postion: " + QString::number(err._gpos,'g',6) +"%");  
  _gbw->getTextBox()->append("Component Wise - ");
  _gbw->getTextBox()->append("                 X: " + QString::number(err._lpos[0],'g',6)+"%"); 
  _gbw->getTextBox()->append("                 Y: " + QString::number(err._lpos[1],'g',6)+"%");
  _gbw->getTextBox()->append("                 Z: " + QString::number(err._lpos[2],'g',6)+"%");
   _gbw->getTextBox()->append("    ");
  _gbw->getTextBox()->append("Orientation: " + QString::number(err._grot,'g',6)+"%");  
  _gbw->getTextBox()->append("Component Wise - "); 
  _gbw->getTextBox()->append( "                 QX: " + QString::number(err._lrot.V()[0],'g',6)+"%");
  _gbw->getTextBox()->append("                 QY: " + QString::number(err._lrot.V()[1],'g',6)+"%");  
  _gbw->getTextBox()->append("                 QZ: " + QString::number(err._lrot.V()[2],'g',6)+"%");
  _gbw->getTextBox()->append("                 QW: " + QString::number(err._lrot.S(),'g',6)+"%");
  _gbw->getTextBox()->append("    ");




}


void
Results::printTitle(const char *title)
{
 // we have ther error values so print them out
  _gbw->getTextBox()->append("   ");
  _gbw->getTextBox()->append(title);
  _gbw->getTextBox()->append("------------------------------");
  _gbw->getTextBox()->append("File: " + _gbw->getCPanel()->getCurFile()->text());
}

void
Results::printRootMSE(ErrorNum err)
{
  printTitle("ROOT MEAN SQUARE ERROR RESULTS");

  printBasicMaterial();
  
  printSingleReport(err);

}



void
Results::printLInf(ErrorNum err)
{

  printTitle("L-INFINITY(MAX NORM) ERROR RESULTS");

  printBasicMaterial();

  printSingleReport(err); 
 
}


void
Results::printPB(ErrorNum err)
{
 printTitle("PERCENT BETTER THAN RANDOM-WALK FORECAST");

 printBasicMaterial();

 printPctReport(err);

}


// methods for computing and printing results in varying parameter testing

void
Results::displayParamEandG()
{

 // switch on error type
  switch(_etype) {
    case tRootMSE: {
      switch (_gbw->getPredType()) {
       case tLocalPoly: {
         if ((_gbw->getLPolyDialog()->getLPType() == tlagrangian) ||
             (_gbw->getLPolyDialog()->getLPType() == tcspline))
           print1D(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS");
         else if (_gbw->getLPolyDialog()->getLPType() == tlsquares)
           print2D(_errlistrms,"ROOT MEAN SQUARE ERROR RESULTS");
         break;
       }
       case tGrey: {
          print1D(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS");
          break;
	}
       case tExpSmooth: {
          print1D(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS",4);       
	  break;
       }
       case tCI:
       case tKalman: {
           print1D(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS", 2);
          break;
	}
       default:
        ;
      }
 
     break;
    }
    case tLInfinity: {
      switch (_gbw->getPredType()) {
       case tLocalPoly: {
         if ((_gbw->getLPolyDialog()->getLPType() == tlagrangian) ||
             (_gbw->getLPolyDialog()->getLPType() == tcspline))
           print1D(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS");
         else if (_gbw->getLPolyDialog()->getLPType() == tlsquares)
           print2D(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS");
         break;
       }
      case tGrey: {
          print1D(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS");
          break;
      }
      case tExpSmooth: {
         print1D(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS",4);
	 break;
      }
       case tCI:
       case tKalman: {
          print1D(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS", 2);
          break;
      }
       default:
        ;
      }  

     break;
    }
    case tPctBetter: {
     switch (_gbw->getPredType()) {
       case tLocalPoly: {
         if ((_gbw->getLPolyDialog()->getLPType() == tlagrangian) ||
             (_gbw->getLPolyDialog()->getLPType() == tcspline))
           print1D(_errlistpct, "PERCENT BETTER THAN RANDOM WALK FORECAST", 1);
         else if (_gbw->getLPolyDialog()->getLPType() == tlsquares)
           print2D(_errlistpct, "PERCENT BETTER THAN RANDOM WALK FORECAST", 1);
         break;
       }
        case tGrey: {
          print1D(_errlistpct, "PERCENT BETTER THAN RANDOM WALK FORECAST", 1);
          break;
      }
       case tExpSmooth: {
          print1D(_errlistpct, "PERCENT BETTER THAN RANDOM WALK FORECAST", 5);
	  break;
       }
       case tCI:
       case tKalman: {
          print1D(_errlistpct, "PERCENT BETTER THAN RANDOM WALK FORECAST", 3);
          break;
      }
       default:
        ;
      }  

     break;
    }

    case tChooseAll: {
      // compute and print out all error metrics
	switch (_gbw->getPredType()) {
	case tLocalPoly: {
	  if ((_gbw->getLPolyDialog()->getLPType() == tlagrangian) ||
	      (_gbw->getLPolyDialog()->getLPType() == tcspline)) {
	    print1D(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS");
	    print1D(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS");
	    print1D(_errlistpct, "PERCENT BETTER THAN RANDOM WALK FORECAST", 1);
          }
	  else if (_gbw->getLPolyDialog()->getLPType() == tlsquares) {
	    print2D(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS");
            print2D(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS");
            print2D(_errlistpct, "PERCENT BETTER THAN RANDOM WALK FORECAST", 1);
          }
	  break;
	}
        case tGrey: {
	  print1D(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS");
	  print1D(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS");
	  print1D(_errlistpct, "PERCENT BETTER THAN RANDOM WALK FORECAST", 1);
	  break;
	}
	case tExpSmooth: {
          print1D(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS",4);
	  print1D(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS", 4);
	  print1D(_errlistpct, "PERCENT BETTER THAN RANDOM WALK FORECAST",5);
          break;
        }
	case tCI:
        case tKalman: {
	  print1D(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS",2);
	  print1D(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS", 2);
	  print1D(_errlistpct, "PERCENT BETTER THAN RANDOM WALK FORECAST",3);
	  break;
	}
	default:
	  ;
      }  
     break;
    }
  default:
    cerr << "We have a problem in Results::displaySingleError()!!!" << endl;
  }
}

void
Results:: print1D(FIGarray<ErrorNum> errlist, const char *title, int pct)
{
  // first we print out to the text window
  printTitle(title);
  printBasicMaterial();
  
  if (pct == 1)
    _gbw->getTextBox()->append("Window Size      Position        Orientation");
  else if (pct == 0)
    _gbw->getTextBox()->append("Window Size     Position(inches)     Orientation(degrees)");
  else if (pct == 2)
    _gbw->getTextBox()->append("Spectral D      Position(inches)        Orientation(degrees)");
  else if (pct == 3)
    _gbw->getTextBox()->append("Spectral D      Position       Orientation");
  else if (pct == 4)
    _gbw->getTextBox()->append("Alpha     Position(inches)             Orientation(degrees)");
  else if (pct == 5)
    _gbw->getTextBox()->append("Alpha     Position            Orientation");

  int offset = 0;
  if (_gbw->getLPolyDialog()->getLPType() == tcspline)  
    offset = 1; // needed since cubic spline predictor starts at window size 2
  if (_gbw->getPredType() == tGrey)
    offset = 3;  // grey starts at 4

  double alpha = 0.01;
  for (int i = 0; i < errlist.num(); ++i) {
    char buf[50]; 
    if ((pct == 1) || (pct == 3))
      sprintf(buf, "%2d \t\t %3.4lf%% \t %3.4lf%%", i+1+offset, errlist[i]._gpos, errlist[i]._grot);
    else if ((pct == 0) || (pct == 2)) 
      sprintf(buf, "%2d \t\t %5.12lf \t %5.12lf", i+1+offset, 12.0 * errlist[i]._gpos, errlist[i]._grot);
    else if (pct == 4) {
      sprintf(buf, "%1.2lf \t %5.12lf \t %5.12lf", alpha, 12.0 * errlist[i]._gpos, errlist[i]._grot); 
      alpha += 0.01;
    }
    else if (pct == 5) {
      sprintf(buf, "%1.2lf \t %3.4lf%% \t %3.4lf%%", alpha, errlist[i]._gpos, errlist[i]._grot);
      alpha += 0.01;
    }
     _gbw->getTextBox()->append(buf);
  }

  // lets do a graph data if matlab is open
  MATLABEngine *eng = _gbw->getMEngine();
  if (!eng->getEngine()) // if its not there do nothing
    return;

  eng->runCmd("clear");
  // create arrays for our elements
  int num = errlist.num();
  double *erpos = new double[num];
  double *errot = new double[num];
  double *winn  = new double[num];

  alpha = 0.01;
  for (int i = 0; i < num; ++i) {
   if ((pct == 1) || (pct == 3) || (pct == 5))
    erpos[i] = errlist[i]._gpos;
   else
    erpos[i] = 12.0*errlist[i]._gpos;
   errot[i] = errlist[i]._grot;
   if ((pct == 4) || (pct == 5)) {
      winn[i] = alpha;
      alpha += 0.01;
   }
   else 
      winn[i] = (double)i+1.0+(double)offset;
  }
  
  // send data to matlab
  mxArray *EPOS =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxArray *EROT =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxArray *WIN  =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxSetName(EPOS,"erpos");
  mxSetName(EROT,"errot");
  mxSetName(WIN,"win");
  memcpy((void*)mxGetPr(EPOS), (void*)erpos, num*sizeof(double));
  memcpy((void*)mxGetPr(EROT), (void*)errot, num*sizeof(double));
  memcpy((void*)mxGetPr(WIN), (void*)winn, num*sizeof(double));
  eng->sendArray(EPOS); 
  eng->sendArray(EROT); 
  eng->sendArray(WIN); 
  mxDestroyArray(EPOS);
  mxDestroyArray(EROT);
  mxDestroyArray(WIN);
  
  // create the plots
  eng->runCmd(QString("figure"));
  eng->runCmd(QString("plot(win,erpos, 'r-')"));
  eng->runCmd(QString("title('") + QString(title) + QString("')"));
  if ((pct == 0) || (pct == 2)) 
    eng->runCmd(QString("xlabel('Win/Degree')"));
  else if ((pct == 1) || (pct == 3))
    eng->runCmd(QString("xlabel('Spectral Density')"));
  else 
    eng->runCmd(QString("xlabel('Alpha')"));


  if ((pct == 1) || (pct == 3) || (pct == 5))
    eng->runCmd(QString("ylabel('Pct in Position')"));
  else
    eng->runCmd(QString("ylabel('Error (inches)')"));

  eng->runCmd(QString("figure"));
  eng->runCmd(QString("plot(win,errot, 'b-')"));
  eng->runCmd(QString("title('") + QString(title) + QString("')"));    
   if ((pct == 0) || (pct == 2)) 
    eng->runCmd(QString("xlabel('Win/Degree')"));
   else if ((pct == 1) || (pct == 3))
    eng->runCmd(QString("xlabel('Spectral Density')"));
   else 
    eng->runCmd(QString("xlabel('Alpha')"));

  if ((pct == 1) || (pct == 3) || (pct == 5))
    eng->runCmd(QString("ylabel('Pct in Rotation')"));
  else
    eng->runCmd(QString("ylabel('Error (degrees)')"));
  
  delete [] erpos;
  delete [] errot;
  delete [] winn;
}

void
Results:: print2D(FIGarray<ErrorNum> errlist, const char *title, int pct)
{
  // first we print out to the text window
  printTitle(title);
  printBasicMaterial();


  if (pct)
    _gbw->getTextBox()->append("                      Window Size vs. Polynomial Degree");
  else
    _gbw->getTextBox()->append("                      Window Size vs. Polynomial Degree  (Position in inches)");
  
  int winmax = _gbw->getLPolyDialog()->getPWindow();
  int degree = _gbw->getLPolyDialog()->getPDegree();

  char buf[400];
  
  for (int i = 1; i <= degree; ++i)
    sprintf(buf+((i-1)*20), "%20d", i);
  _gbw->getTextBox()->append(buf);
 
  
  int cnt = 0;
  int dummy = -1;
  for(int i = 2; i <= winmax; ++i) {
    char buf2[1024];
    sprintf(buf2, "%2d", i);      
    for(int j = 1; j <= degree; ++j) {
      if (j < i)  // we have a record for win i, deg j
	if (pct)
          sprintf(buf2+2+((j-1)*20), "%19.10lf%%", errlist[cnt++]._gpos);
	else
	  sprintf(buf2+2+((j-1)*20), "%20.10lf", 12.0*errlist[cnt++]._gpos);
      else // put in a filler
	sprintf(buf2+2+((j-1)*20), "%20d", dummy);
    }
    _gbw->getTextBox()->append(buf2);
  }
 
  // now for orientation
  _gbw->getTextBox()->append("   ");

  if (pct)
     _gbw->getTextBox()->append("                      Window Size vs. Polynomial Degree");
  else
    _gbw->getTextBox()->append("                      Window Size vs. Polynomial Degree  (Orientation in degrees)");
   
   for (int i = 1; i <= degree; ++i)
    sprintf(buf+((i-1)*20), "%20d", i);
   _gbw->getTextBox()->append(buf);

  cnt = 0;
  for(int i = 2; i <= winmax; ++i) {
    char buf2[1024];
    sprintf(buf2, "%2d", i);      
    for(int j = 1; j <= degree; ++j) {
      if (j < i)  // we have a record for win i, deg j
       if (pct)
	 sprintf(buf2+2+((j-1)*20), "%19.10lf%%", errlist[cnt++]._grot);
       else
	 sprintf(buf2+2+((j-1)*20), "%20.10lf", errlist[cnt++]._grot);
      else // put in a filler
	sprintf(buf2+2+((j-1)*20), "%20d", dummy);
    }
    _gbw->getTextBox()->append(buf2);
  }

  // matlab output if needed
   MATLABEngine *eng = _gbw->getMEngine();
  if (!eng->getEngine()) // if its not there do nothing
    return;

  eng->runCmd("clear");
  // create arrays for our elements
  double *m = new double[winmax-1];
  double *n = new double[degree];
  double *matp = new double[degree*(winmax-1)];
  double *matq = new double[degree*(winmax-1)];

  // load the data
  cnt = 0;
  int bnt = 0;
  for(int i = 2; i <= winmax; ++i) {
    m[i-2] = (double)i;
    for(int j = 1; j <= degree; ++j) {
     n[j-1] = (double)j;
     if (j < i) {
       if (pct)
         matp[bnt] = errlist[cnt]._gpos;
       else
	 matp[bnt] = errlist[cnt]._gpos * 12.0;
       matq[bnt] = errlist[cnt++]._grot;    
     }
     else {
       matp[bnt] = -1.0;
       matq[bnt] = -1.0;    
     }
     cerr << matq[bnt] << " ";
     ++bnt;
    }
     cerr << endl;
  }
  
  //char mbuf[2048];
  //engOutputBuffer(eng->getEngine(), mbuf, 2048);
  // send data to matlab
  mxArray *EPOS =  mxCreateDoubleMatrix(degree, winmax-1, mxREAL);
  mxArray *EROT =  mxCreateDoubleMatrix(degree, winmax-1, mxREAL);
  mxArray *WIN  =  mxCreateDoubleMatrix(1, winmax-1, mxREAL);
  mxArray *DEG  =  mxCreateDoubleMatrix(1, degree, mxREAL);
  mxSetName(EPOS,"erpos");
  mxSetName(EROT,"errot");
  mxSetName(WIN,"win");
  mxSetName(DEG,"deg");
  memcpy((void*)mxGetPr(WIN), (void*)m, (winmax-1)*sizeof(double));
  memcpy((void*)mxGetPr(DEG), (void*)n, degree*sizeof(double)); 
  memcpy((void*)mxGetPr(EPOS), (void*)matp, degree*(winmax-1)*sizeof(double));
  memcpy((void*)mxGetPr(EROT), (void*)matq, degree*(winmax-1)*sizeof(double)); 

  eng->sendArray(EPOS); 
  eng->sendArray(EROT); 
  eng->sendArray(WIN); 
  eng->sendArray(DEG);

  // eng->runCmd(QString("errot'"));
  //cerr << mbuf << endl;
  mxDestroyArray(EPOS);
  mxDestroyArray(EROT);
  mxDestroyArray(WIN);
  mxDestroyArray(DEG);

  // create the plots
  eng->runCmd(QString("figure"));
  eng->runCmd(QString("surf(deg,win,erpos')"));
  eng->runCmd(QString("title('") + QString(title) + QString("')"));
  eng->runCmd(QString("xlabel('Degree')"));
  eng->runCmd(QString("ylabel('Window Size')"));
  if (pct)
    eng->runCmd(QString("zlabel('Pct in Position')"));
  else
    eng->runCmd(QString("zlabel('Error (inches)')"));

  eng->runCmd(QString("figure"));
  eng->runCmd(QString("surf(deg,win,errot')"));
  eng->runCmd(QString("title('") + QString(title) + QString("')"));
  eng->runCmd(QString("xlabel('Degree')"));
  eng->runCmd(QString("ylabel('Window Size')"));
  if (pct)
   eng->runCmd(QString("zlabel('Pct in Rotation')"));  
  else
    eng->runCmd(QString("zlabel('Error (degrees)')"));  

  // delete memory
  delete [] m;
  delete [] n;
  delete [] matp;
  delete [] matq;

}

// methods for printing and graphing prediction tests over different
// sampling rates

void
Results::sRateResults(int ntests)
{
  switch(_etype) {
    case tRootMSE: {
       printSRates(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS OVER DIFFERENT SAMPLING RATES", ntests);
       break;
    }
    case tLInfinity: {
       printSRates(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS OVER DIFFERENT SAMPLING RATES", ntests);
      break;
    }
    case tPctBetter: {
      printSRates(_errlistpct, "PCT BETTER THAN RANDOM-WALK FORECAST OVER DIFFERENT SAMPLING RATES", ntests, 1);
      break;
    }
    case tChooseAll: {
        printSRates(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS OVER DIFFERENT SAMPLING RATES", ntests);
	printSRates(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS OVER DIFFERENT SAMPLING RATES", ntests);
          printSRates(_errlistpct, "PCT BETTER THAN RANDOM-WALK FORECAST OVER DIFFERENT SAMPLING RATES", ntests, 1);
        break;
    }
    default:  
      cerr << "Problem in sRateResults!!!" << endl;
  }

}


void
Results::printSRates(FIGarray<ErrorNum> errlist, const char *title, int ntests, int pct)
{
  // first we print out to the text window
  printTitle(title);
  printBasicMaterial();
 _gbw->getTextBox()->append("  ");

  if (pct)
    _gbw->getTextBox()->append("Sampling Rate(Hz)         Position               Orientation");
  else
    _gbw->getTextBox()->append("Sampling Rate(Hz)         Position(inches)               Orientation(degrees)");

 // get the original sampling rate
 double srate = _gbw->getCPanel()->getOrigSRate();

 for (int i = 0; i < errlist.num(); ++i) {
    char buf[100]; 
    if (pct)
        sprintf(buf, "%4.7lf \t\t  %3.6lf%% \t\t %3.6lf%%", srate / (ntests-i), errlist[i]._gpos, errlist[i]._grot);
    else
      sprintf(buf, "%4.7lf \t\t  %5.12lf \t\t %5.12lf", srate / (ntests-i), 12.0*errlist[i]._gpos, errlist[i]._grot);
     _gbw->getTextBox()->append(buf);
  }

   // lets do a graph data if matlab is open
  MATLABEngine *eng = _gbw->getMEngine();
  if (!eng->getEngine()) // if its not there do nothing
    return;

  eng->runCmd("clear");
  // create arrays for our elements
  int num = errlist.num();
  double *erpos = new double[num];
  double *errot = new double[num];
  double *sr  = new double[num];

   for (int i = 0; i < num; ++i) {
    if (pct)
      erpos[i] = errlist[i]._gpos;
    else
      erpos[i] = 12.0*errlist[i]._gpos;
    errot[i] = errlist[i]._grot;
    sr[i] = srate / (ntests-i);
  }

 // send data to matlab
  mxArray *EPOS =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxArray *EROT =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxArray *SR  =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxSetName(EPOS,"erpos");
  mxSetName(EROT,"errot");
  mxSetName(SR,"sr");
  memcpy((void*)mxGetPr(EPOS), (void*)erpos, num*sizeof(double));
  memcpy((void*)mxGetPr(EROT), (void*)errot, num*sizeof(double));
  memcpy((void*)mxGetPr(SR), (void*)sr, num*sizeof(double));
  eng->sendArray(EPOS); 
  eng->sendArray(EROT); 
  eng->sendArray(SR); 
  mxDestroyArray(EPOS);
  mxDestroyArray(EROT);
  mxDestroyArray(SR);
  
  // create the plots
  eng->runCmd(QString("figure"));
  eng->runCmd(QString("plot(sr,erpos, 'r-')"));
  eng->runCmd(QString("title('") + QString(title) + QString("')"));
  eng->runCmd(QString("xlabel('Hertz')"));
  if (pct)
    eng->runCmd(QString("ylabel('Pct in Position')"));
  else
    eng->runCmd(QString("ylabel('Error (inches)')"));

  eng->runCmd(QString("figure"));
  eng->runCmd(QString("plot(sr,errot, 'b-')"));
  eng->runCmd(QString("title('") + QString(title) + QString("')"));
  eng->runCmd(QString("xlabel('Hertz')"));
  if (pct)
    eng->runCmd(QString("ylabel('Pct in Rotation')"));
  else
    eng->runCmd(QString("ylabel('Error (degrees)')"));
  
  delete [] erpos;
  delete [] errot;
  delete [] sr;

}


// methods for printing results of tests over different prediction intervals

void
Results::pIntResults(int min, int max, int skip)
{
 switch(_etype) {
    case tRootMSE: {
       printPInt(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS OVER DIFFERENT PREDICTION TIMES", min, max, skip);
       break;
    }
    case tLInfinity: {
       printPInt(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS OVER DIFFERENT PREDICTION TIMES", min, max, skip);
      break;
    }
    case tPctBetter: {
      printPInt(_errlistpct, "PCT BETTER THAN RANDOM WALK FORECAST OVER DIFFERENT PREDICTION TIMES", min, max, skip, 1);
      break;
    }
    case tChooseAll: {
        printPInt(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS OVER DIFFERENT PREDICTION TIMES", min, max, skip);
	printPInt(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS OVER DIFFERENT PREDICTION TIMES", min, max, skip);
	printPInt(_errlistpct, "PCT BETTER THAN RANDOM WALK FORECAST OVER DIFFERENT PREDICTION TIMES", min, max, skip, 1);
        break;
    }
    default:  
      cerr << "Problem in sRateResults!!!" << endl;
  }

}


void
Results::printPInt(FIGarray<ErrorNum> errlist, const char *title, int min,
                  int max, int skip, int pct)

{

 // first we print out to the text window
  printTitle(title);
  printBasicMaterial(1);

 _gbw->getTextBox()->append("  ");
 if (pct)
   _gbw->getTextBox()->append("Prediction Time(ms)       Position               Orientation");
 else 
   _gbw->getTextBox()->append("Prediction Time(ms)       Position(inches)               Orientation(degrees)");

 // get the original sampling rate
 double srate = _gbw->getCPanel()->getOrigSRate();

 for (int i = 0; i < errlist.num(); ++i) {
    char buf[100]; 
    int ldtime = min + (skip * i);
    double pd = ldtime * (1.0 / _gbw->getCPanel()->getOrigSRate());
    if (pct)
      sprintf(buf, "%4.7lf \t\t  %3.6lf%% \t\t %3.6lf%%", 1000*pd, errlist[i]._gpos,
            errlist[i]._grot);
    else
      sprintf(buf, "%4.7lf \t\t  %5.12lf \t\t %5.12lf", 1000*pd, 12.0*errlist[i]._gpos,
            errlist[i]._grot);
     _gbw->getTextBox()->append(buf);
  }

   // lets do a graph data if matlab is open
  MATLABEngine *eng = _gbw->getMEngine();
  if (!eng->getEngine()) // if its not there do nothing
    return;

  eng->runCmd("clear");
  // create arrays for our elements
  int num = errlist.num();
  double *erpos = new double[num];
  double *errot = new double[num];
  double *pint  = new double[num];

   for (int i = 0; i < num; ++i) {
    if (pct)
      erpos[i] = errlist[i]._gpos;
    else
      erpos[i] = 12.0*errlist[i]._gpos;
    errot[i] = errlist[i]._grot;
    int ldtime = min + (skip * i);
    double pd = ldtime * (1.0 / _gbw->getCPanel()->getOrigSRate());
    pint[i] =   1000.0 * pd;
  }

 // send data to matlab
  mxArray *EPOS =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxArray *EROT =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxArray *PINT  =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxSetName(EPOS,"erpos");
  mxSetName(EROT,"errot");
  mxSetName(PINT,"pint");
  memcpy((void*)mxGetPr(EPOS), (void*)erpos, num*sizeof(double));
  memcpy((void*)mxGetPr(EROT), (void*)errot, num*sizeof(double));
  memcpy((void*)mxGetPr(PINT), (void*)pint, num*sizeof(double));
  eng->sendArray(EPOS); 
  eng->sendArray(EROT); 
  eng->sendArray(PINT); 
  mxDestroyArray(EPOS);
  mxDestroyArray(EROT);
  mxDestroyArray(PINT);
  
  // create the plots
  eng->runCmd(QString("figure"));
  eng->runCmd(QString("plot(pint,erpos, 'r-')"));
  eng->runCmd(QString("title('") + QString(title) + QString("')"));
  eng->runCmd(QString("xlabel('Milliseconds')"));
  if (pct)
    eng->runCmd(QString("ylabel('Pct in Position')"));
  else 
    eng->runCmd(QString("ylabel('Error (inches)')"));

  eng->runCmd(QString("figure"));
  eng->runCmd(QString("plot(pint,errot, 'b-')"));
  eng->runCmd(QString("title('") + QString(title) + QString("')"));
  eng->runCmd(QString("xlabel('Milliseconds')"));
  if (pct)
    eng->runCmd(QString("ylabel('Pct in Rotation')"));
  else
    eng->runCmd(QString("ylabel('Error (degrees)')"));
  
  delete [] erpos;
  delete [] errot;
  delete [] pint;

}


// methods for printing results of tests over different noise variances

void
Results::errResults(int step, int max, int noisenum) 
{ 
 switch(_etype) {
    case tRootMSE: {
       printErr(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS OVER DIFFERENT NOISE VARIANCES", step, max, noisenum);
       break;
    }
    case tLInfinity: {
       printErr(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS OVER DIFFERENT NOISE VARIANCES", step, max, noisenum);
      break;
    }
    case tPctBetter: {
      printErr(_errlistpct, "PCT BETTER THAN RANDOM WALK FORECAST OVER DIFFERENT NOISE VARIANCES", step, max, noisenum, 1);
      break;
    }
    case tChooseAll: {
        printErr(_errlistrms, "ROOT MEAN SQUARE ERROR RESULTS OVER DIFFERENT NOISE VARIANCES", step, max, noisenum);
	printErr(_errlistinf, "L-INFINITY(MAX NORM) ERROR RESULTS OVER DIFFERENT NOISE VARIANCES", step, max, noisenum);
	printErr(_errlistpct, "PCT BETTER THAN RANDOM WALK FORECAST OVER DIFFERENT NOISE VARIANCES", step, max, noisenum, 1);
        break;
    }
    default:  
      cerr << "Problem in errResults!!!" << endl;
  }

}



void
Results::printErr(FIGarray<ErrorNum> errlist, const char *title, int step,
                  int max, int noisenum, int pct)

{

 // first we print out to the text window
  printTitle(title);
  printBasicMaterial(0);

 _gbw->getTextBox()->append("  ");
 if (pct)
   _gbw->getTextBox()->append("Noise Variance            Position               Orientation");
 else 
   _gbw->getTextBox()->append("Noise Variance            Position(inches)               Orientation(degrees)");


 for (int i = 0; i < errlist.num(); ++i) {
    char buf[100]; 
    double noise = (double)((i * step) + step) / (double) noisenum;
    if (pct)
      sprintf(buf, "%4.7lf \t\t  %3.6lf%% \t\t %3.6lf%%", noise, errlist[i]._gpos,
            errlist[i]._grot);
    else
      sprintf(buf, "%4.7lf \t\t  %5.12lf \t\t %5.12lf", noise, 12.0*errlist[i]._gpos,
            errlist[i]._grot);
     _gbw->getTextBox()->append(buf);
  }

   // lets do a graph data if matlab is open
  MATLABEngine *eng = _gbw->getMEngine();
  if (!eng->getEngine()) // if its not there do nothing
    return;

  eng->runCmd("clear");
  // create arrays for our elements
  int num = errlist.num();
  double *erpos = new double[num];
  double *errot = new double[num];
  double *nvar  = new double[num];

   for (int i = 0; i < num; ++i) {
    if (pct)
      erpos[i] = errlist[i]._gpos;
    else
      erpos[i] = 12.0*errlist[i]._gpos;
    errot[i] = errlist[i]._grot;
    double noise = (double)((i * step) + step) / (double) noisenum;
    nvar[i] = noise;  
  }

 // send data to matlab
  mxArray *EPOS =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxArray *EROT =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxArray *NVAR  =  mxCreateDoubleMatrix(1, num, mxREAL);
  mxSetName(EPOS,"erpos");
  mxSetName(EROT,"errot");
  mxSetName(NVAR,"nvar");
  memcpy((void*)mxGetPr(EPOS), (void*)erpos, num*sizeof(double));
  memcpy((void*)mxGetPr(EROT), (void*)errot, num*sizeof(double));
  memcpy((void*)mxGetPr(NVAR), (void*)nvar, num*sizeof(double));
  eng->sendArray(EPOS); 
  eng->sendArray(EROT); 
  eng->sendArray(NVAR); 
  mxDestroyArray(EPOS);
  mxDestroyArray(EROT);
  mxDestroyArray(NVAR);
  
  // create the plots
  eng->runCmd(QString("figure"));
  eng->runCmd(QString("plot(nvar,erpos, 'r-')"));
  eng->runCmd(QString("title('") + QString(title) + QString("')"));
  eng->runCmd(QString("xlabel('Noise Variance')"));
  if (pct)
    eng->runCmd(QString("ylabel('Pct in Position')"));
  else 
    eng->runCmd(QString("ylabel('Error (inches)')"));

  eng->runCmd(QString("figure"));
  eng->runCmd(QString("plot(nvar,errot, 'b-')"));
  eng->runCmd(QString("title('") + QString(title) + QString("')"));
  eng->runCmd(QString("xlabel('Noise Variance')"));
  if (pct)
    eng->runCmd(QString("ylabel('Pct in Rotation')"));
  else
    eng->runCmd(QString("ylabel('Error (degrees)')"));
  
  delete [] erpos;
  delete [] errot;
  delete [] nvar;

}
