// This is the main class for computing results and displaying them
// in text and graphics.

#ifndef RESULTS_H
#define RESULTS_H

#include "types.H"
#include "list.H"
#include "math3.H"

class GlobalWidget;
class PredictionRec;
class UMotionRec;
class Results;
class Pose;


// this class is in charge of taking the prediction record(s) and the user motion
// record(s) to perform error analysis and reports results

class ErrorNum
{
  public: 
   ErrorNum() : _gpos(0.0), _grot(0.0), _lpos(-1.0,-1.0,-1.0), 
               _lrot(-1.0,-1.0,-1.0,-1.0) { }
   virtual ~ErrorNum() { }

  public:
    double _gpos;
    double _grot;
    Vec3d  _lpos;
    Quat   _lrot;
};


class Results
{
  public:
    Results(GlobalWidget *gbw);
    virtual ~Results();

    void reload();
    // clear data structure
    void clear();
    void clearPRec(); 

    void insertPRec(PredictionRec *prec) { _preclist += prec; }
    void insertURec(UMotionRec *urec)    { _ureclist += urec; }
    void errorAnalysis(); 
    void computeSingleError();

   // helper functions for this class
   void displaySingleGraphics();
   void displaySingleError();    
   void plotErrorGraphs();

   void displayParamEandG();
   void sRateResults(int ntests);
   void pIntResults(int min, int max, int skip); 
   void errResults(int step, int max, int noisemax);   

   void plotSignal(const char *trues, const char *preds, const char *corrs,
                   const char *subplot,
		   const char *plottrue, const char *plotpred, const char *plotdot,
                   const char *corsig,  const char *title, 
		   const char *xlabel,
		   const char *ylabel, int numpred, int numtrue, int numcorr,
		   double *truesig, double *predsig, double *corrupted);

   void plotError(const char *name, const char *subplot, const char *plot,
                  const char *title, const char *xlabel, const char *ylabel,
                  int num, double *diff);


   ErrorNum rootMSE(int i);
   ErrorNum lInfinity(int i);
   ErrorNum pctBetter(int i);

  // print routines
   void printRootMSE(ErrorNum err);
   void printLInf(ErrorNum err);
   void printPB(ErrorNum err);
   void printBasicMaterial(int nopint = 0);
   void printSingleReport(ErrorNum err);
   void printPctReport(ErrorNum err);
   void printTitle(const char *title);

   void print1D(FIGarray<ErrorNum> errlist, const char *title, int pct = 0);
   void print2D(FIGarray<ErrorNum> errlist, const char *title, int pct = 0);

   void printSRates(FIGarray<ErrorNum> errlist, const char *title, int ntests, 
                    int pct = 0); 
   void printPInt(FIGarray<ErrorNum> errlist, const char *title, int min,
                  int max, int skip, int pct = 0);  
   void printErr(FIGarray<ErrorNum> errlist, const char *title, int step,
                 int max, int noisenum, int pct = 0);

  protected:
    GlobalWidget             *_gbw;
    FIGarray<PredictionRec *> _preclist;
    FIGarray<UMotionRec *>    _ureclist;
    TestType                  _ttype;
    ErrorType                 _etype;
  
    // hold error lists
    FIGarray<ErrorNum>        _errlistrms;
    FIGarray<ErrorNum>        _errlistinf;
    FIGarray<ErrorNum>        _errlistpct;
};

#endif
